/**
 * Copyright (c) 2012 EBM WebSourcing, 2012-2013 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see <http://www.gnu.org/licenses/>
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.admin.api.artifact.lifecycle;

import java.net.URL;

import org.ow2.petals.admin.api.exception.ArtifactAdministrationException;
import org.ow2.petals.admin.api.exception.ArtifactDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotDeployedException;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactStartedException;
import org.ow2.petals.admin.api.exception.ArtifactStoppedException;

public interface ArtifactLifecycle {

    /**
     * Install/deploy the artifact
     * 
     * @throws ArtifactDeployedEXception
     *             The artifact is already deployed.
     * @throws ArtifactAdministrationException
     *             An error occurs during the artifact deployment.
     */
    public void deploy(URL artifactUrl) throws ArtifactDeployedException,
            ArtifactAdministrationException;

    /**
     * Start the artifact
     * 
     * @throws ArtifactStartedException
     *             The artifact is already started
     * @throws ArtifactNotDeployedException
     *             The artifact is not deployed
     * @throws ArtifactNotFoundException
     *             The artifact can not be found
     * @throws ArtifactAdministrationException
     *             An error occurs during the artifact start.
     */
    public void start() throws ArtifactStartedException, ArtifactNotDeployedException,
            ArtifactNotFoundException, ArtifactAdministrationException;

    /**
     * Stop the artifact
     * 
     * @throws ArtifactStoppedException
     *             The artifact is already stopped
     * @throws ArtifactNotDeployedException
     *             The artifact is not deployed
     * @throws ArtifactNotFoundException
     *             The artifact can not be found
     * @throws ArtifactAdministrationException
     *             An error occurs during the artifact stop.
     * 
     */
    public void stop() throws ArtifactStoppedException, ArtifactNotDeployedException,
            ArtifactNotFoundException, ArtifactAdministrationException;

    /**
     * Undeploy the artifact
     * 
     * @throws ArtifactStartedException
     *             The artifact is started
     * @throws ArtifactNotFoundException
     *             The artifact can not be found
     * @throws ArtifactAdministrationException
     *             An error occurs during the artifact stop.
     */
    public void undeploy() throws ArtifactStartedException, ArtifactNotFoundException,
            ArtifactAdministrationException;
    
    /**
     * Update the state of the artifact (if there is one)
     */
    public void updateState() throws ArtifactAdministrationException;
}
